module Fastlane
  module Actions
    module SharedValues
      ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL = :ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL
    end
    class AndroidChangePackageIdentifierAction < Action
      def self.run(params)
        require 'nokogiri'

        newIdentifier = params[:newIdentifier]
        appendToIdentifier = nil
        manifest = params[:manifest]

        if newIdentifier.to_s.strip.length == 0
          appendToIdentifier = params[:appendToIdentifier]

          if appendToIdentifier.to_s.strip.length == 0
            UI.error("Must provide either newIdentifier or appendToIdentifier")
          end
        end


        doc = File.open(manifest) { |f|
          @doc = Nokogiri::XML(f)

          original = nil

          @doc.css("manifest").each do |response_node|
            original = response_node["package"]

            if newIdentifier.to_s.strip.length != 0
              response_node["package"] = newIdentifier
            else
              response_node["package"] = "#{original}.#{appendToIdentifier}"
            end
            UI.message("Updating package identifier to: #{response_node["package"]}")
          end

          Actions.lane_context[SharedValues::ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL] = original

          File.write(manifest, @doc.to_xml)
        }
      end

      def self.description
        "Change the package identifier in the AndroidManifest.xml file. Can revert as well."
      end

      def self.authors
        ["MaximusMcCann"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        # Optional:
        "Change the package identifier in the AndroidManifest.xml file. Can revert as well."
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :newIdentifier,
                                  env_name: "",
                               description: "The new package identifier for the app. Trumps appendToIdentifier",
                                  optional: true,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :appendToIdentifier,
                                  env_name: "",
                               description: "The text to append to the package identifier. adds a (.) intitally",
                                  optional: true,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :manifest,
                                  env_name: "",
                               description: "Optional custom location for AndroidManifest.xml",
                                  optional: false,
                                      type: String,
                             default_value: "app/src/main/AndroidManifest.xml")
        ]
      end

      def self.output
        [
          ['ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL', 'The original package identifier.']
        ]
      end

      def self.is_supported?(platform)
        platform == :android
      end
    end

    class AndroidChangePackageIdentifierRevertAction < Action
      def self.run(params)
        require 'nokogiri'

        original = Actions.lane_context[SharedValues::ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL]

        if original.to_s.strip.length != 0
          manifest = params[:manifest]
        else
          UI.error("no string for ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL.  Have you run android_change_package_identifier?")
        end

        doc = File.open(manifest) { |f|
          @doc = Nokogiri::XML(f)

          @doc.css("manifest").each do |response_node|
            response_node["package"] = original
            UI.message("Reverting package identifier to: #{original}")
          end

          File.write(manifest, @doc.to_xml)
        }

      end

      def self.description
        "Reverts the manifest's package identifier from ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL"
      end

      def self.authors
        ["MaximusMcCann"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        "Reverts the manifest's package identifier from ANDROID_CHANGE_PACKAGE_IDENTIFIER_ORIGINAL"
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :manifest,
                                  env_name: "",
                               description: "Optional custom location for AndroidManifest.xml",
                                  optional: false,
                                      type: String,
                             default_value: "app/src/main/AndroidManifest.xml")
        ]
      end

      def self.is_supported?(platform)
        platform == :android
      end
    end
  end
end
